#ifndef __OOL_XSIZE_H__
#define __OOL_XSIZE_H__

/*===========================================================================*/
/* OOL ------------------- the Open Object Library ------------- OS/2 Beta 1 */
/*===========================================================================*/
/*                              class: XSize                                 */
/*                       derived from: XObject                               */
/*                        last update: 3/97                                  */
/*                      programmed by: Stefan von Brauk (sbrauk@gwdg.de)     */
/*===========================================================================*/



#include "xobject.h"
#include "string.h"

class _export_ XSize: public XObject
{
	private:
		LONG width, height;
	public:
		XSize(LONG x=0, LONG y=0) {width = x; height = y; }
		LONG GetWidth() const { return width; } 
		LONG GetHeight() const { return height; }
		void SetWidth( LONG x) { width = x; }
		void SetHeight( LONG y ) { height = y; }
		XSize operator =(const XSize& s)
         {
            width=s.width;
            width=s.height;
            return *this;
         }
};

#endif
